@@ -55,7 +55,7 @@ namespace gensios {
 
 		    if (event == GENSIO_EVENT_SER_SIGNATURE) {
 			std::vector<unsigned char> sig(buf, buf + *buflen);
-			scb->signature(sig);
+			scb->signature(std::move(sig));
 			return 0;
 		    }
 
@@ -160,7 +160,7 @@ namespace gensios {
 
 		case GENSIO_EVENT_PASSWORD_VERIFY: {
 		    std::string pwstr((char *) buf);
-		    return cb->password_verify(pwstr);
+		    return cb->password_verify(std::move(pwstr));
 		}
 
 		case GENSIO_EVENT_REQUEST_PASSWORD: {
@@ -179,7 +179,7 @@ namespace gensios {
 
 		case GENSIO_EVENT_2FA_VERIFY: {
 		    std::vector<unsigned char> val(buf, buf + *buflen);
-		    return cb->verify_2fa(val);
+		    return cb->verify_2fa(std::move(val));
 		}
 
 		case GENSIO_EVENT_REQUEST_2FA: {
@@ -199,6 +199,27 @@ namespace gensios {
 		    *((unsigned char **) buf) = rbuf;
 		    return 0;
 		}
+
+		case GENSIO_EVENT_PARMLOG: {
+		    struct gensio_parmlog_data *d =
+			(struct gensio_parmlog_data *) buf;
+		    va_list argcopy;
+		    va_copy(argcopy, d->args);
+		    size_t len = vsnprintf(NULL, 0, d->log, argcopy);
+		    va_end(argcopy);
+		    std::string outstr(len + 1, '\0');
+		    vsnprintf(&outstr[0], len + 1, d->log, d->args);
+		    cb->parmlog(outstr);
+		    break;
+		}
+
+		case GENSIO_EVENT_WIN_SIZE: {
+		    unsigned int height = 0, width = 0;
+
+		    sscanf((char *) buf, "%u:%u", &height, &width);
+		    cb->win_size(height, width);
+		    break;
+		}
 		}
 		return GE_NOTSUP;
 	    } catch (std::exception &e) {
@@ -332,6 +353,28 @@ namespace gensios {
 	return g;
     }
 
+    static int init_gensio_eventh(struct gensio *io, void *user_data,
+				  int event, int err,
+				  unsigned char *buf, gensiods *buflen,
+				  const char *const *auxdata)
+    {
+	Event *cb = static_cast<Event *>(user_data);
+	struct gensio_parmlog_data *p;
+
+	if (event != GENSIO_EVENT_PARMLOG)
+	    return GE_NOTSUP;
+
+	p = (struct gensio_parmlog_data *) buf;
+	va_list argcopy;
+	va_copy(argcopy, p->args);
+	size_t len = vsnprintf(NULL, 0, p->log, argcopy);
+	va_end(argcopy);
+	std::string outstr(len, '\0');
+	vsnprintf(&outstr[0], len + 1, p->log, p->args);
+	cb->parmlog(outstr);
+	return 0;
+    }
+
     Gensio *
     gensio_alloc(std::string str, Os_Funcs &o, Event *cb)
     {
@@ -339,7 +382,7 @@ namespace gensios {
 	int err;
 	Gensio *g;
 
-	err = str_to_gensio(str.c_str(), o, NULL, NULL, &io);
+	err = str_to_gensio(str.c_str(), o, init_gensio_eventh, cb, &io);
 	if (err)
 	    throw gensio_error(err);
 	g = gensio_alloc(io, o, cb);
@@ -355,7 +398,7 @@ namespace gensios {
 	Gensio *g;
 
 	err = str_to_gensio_child(child->get_gensio(), str.c_str(), o,
-				  NULL, NULL, &io);
+				  init_gensio_eventh, cb, &io);
 	if (err)
 	    throw gensio_error(err);
 	g = gensio_alloc(io, o, cb);
@@ -771,7 +814,7 @@ namespace gensios {
 	Serial_Op_Sig_Done *done = static_cast<Serial_Op_Sig_Done *>(cb_data);
 	std::vector<unsigned char> sigv(sig, sig + len);
 
-	done->serial_op_sig_done(err, sigv);
+	done->serial_op_sig_done(err, std::move(sigv));
     }
 
     void Serial_Gensio::signature(const std::vector<unsigned char> sig,
@@ -1046,6 +1089,19 @@ namespace gensios {
 		    break;
 		}
 
+		case GENSIO_ACC_EVENT_PARMLOG: {
+		    struct gensio_parmlog_data *d =
+			(struct gensio_parmlog_data *) data;
+		    va_list argcopy;
+		    va_copy(argcopy, d->args);
+		    size_t len = vsnprintf(NULL, 0, d->log, argcopy);
+		    va_end(argcopy);
+		    std::string outstr(len + 1, '\0');
+		    vsnprintf(&outstr[0], len + 1, d->log, d->args);
+		    cb->parmlog(outstr);
+		    break;
+		}
+
 		case GENSIO_ACC_EVENT_LOG: {
 		    struct gensio_loginfo *l = (struct gensio_loginfo *) data;
 		    va_list argcopy;
@@ -1075,7 +1131,7 @@ namespace gensios {
 			(struct gensio_acc_password_verify_data *) data;
 		    std::string pwstr((char *) p->password);
 		    Gensio g(p->io, a->get_os_funcs());
-		    return cb->password_verify(&g, pwstr);
+		    return cb->password_verify(&g, std::move(pwstr));
 		}
 
 		case GENSIO_ACC_EVENT_REQUEST_PASSWORD: {
@@ -1101,7 +1157,7 @@ namespace gensios {
 		    std::vector<unsigned char> val(p->password,
 					p->password + p->password_len);
 		    Gensio g(p->io, a->get_os_funcs());
-		    return cb->verify_2fa(&g, val);
+		    return cb->verify_2fa(&g, std::move(val));
 		}
 
 		case GENSIO_ACC_EVENT_REQUEST_2FA: {
@@ -1234,6 +1290,26 @@ namespace gensios {
 	return d->a;
     }
 
+    static int init_gensio_acc_eventh(struct gensio_accepter *iacc,
+				      void *user_data, int event, void *data)
+    {
+	Accepter_Event *cb = static_cast<Accepter_Event *>(user_data);
+	struct gensio_parmlog_data *p;
+
+	if (event != GENSIO_ACC_EVENT_PARMLOG)
+	    return GE_NOTSUP;
+
+	p = (struct gensio_parmlog_data *) data;
+	va_list argcopy;
+	va_copy(argcopy, p->args);
+	size_t len = vsnprintf(NULL, 0, p->log, argcopy);
+	va_end(argcopy);
+	std::string outstr(len, '\0');
+	vsnprintf(&outstr[0], len + 1, p->log, p->args);
+	cb->parmlog(outstr);
+	return 0;
+    }
+
     Accepter *gensio_acc_alloc(std::string str, Os_Funcs &o,
 			       Accepter_Event *cb)
     {
@@ -1241,7 +1317,8 @@ namespace gensios {
 	int err;
 	Accepter *a;
 
-	err = str_to_gensio_accepter(str.c_str(), o, NULL, NULL, &acc);
+	err = str_to_gensio_accepter(str.c_str(), o,
+				     init_gensio_acc_eventh, cb, &acc);
 	if (err)
 	    throw gensio_error(err);
 	a = gensio_acc_alloc(acc, o);
@@ -1260,7 +1337,7 @@ namespace gensios {
 
 	err = str_to_gensio_accepter_child(child->get_accepter(),
 					   str.c_str(), o,
-					   NULL, NULL, &acc);
+					   init_gensio_acc_eventh, cb, &acc);
 	if (err)
 	    throw gensio_error(err);
 	a = gensio_acc_alloc(acc, o);
@@ -1429,7 +1506,8 @@ namespace gensios {
     {
 	struct gensio *io;
 	Gensio *g;
-	int err = gensio_acc_str_to_gensio(acc, str.c_str(), NULL, NULL, &io);
+	int err = gensio_acc_str_to_gensio(acc, str.c_str(),
+					   init_gensio_eventh, cb, &io);
 	if (err)
 	    throw gensio_error(err);
 	g = gensio_alloc(io, go, cb);
