@@ -19,6 +19,7 @@
 #include <stdlib.h>
 
 #include <gensio/gensio.h>
+#include <gensio/sergensio.h>
 #include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_time.h>
 #include <gensio/gensio_ax25_addr.h>
@@ -164,6 +165,15 @@ struct xmit_entry {
     struct xmit_entry *next;
 };
 
+enum afskmdm_keytype {
+    KEY_RW, /* Read and write keyon/keyoff values. */
+    KEY_RTS,
+    KEY_RTSINV,
+    KEY_DTR,
+    KEY_DTRINV,
+    KEY_CM108
+};
+
 struct afskmdm_filter {
     struct gensio_filter *filter;
     struct gensio_os_funcs *o;
@@ -174,6 +184,9 @@ struct afskmdm_filter {
 
     int err;
 
+    /* For reporting key errors. */
+    struct gensio_pparm_info p;
+
     bool simplex;
     unsigned int in_nchans;
     unsigned int in_chan;
@@ -216,10 +229,12 @@ struct afskmdm_filter {
     int out_bit_adj;
     unsigned int max_out_bitsize; /* Largest entry we will send. */
     unsigned int out_bit_counter; /* Counter for out_bit_period */
-    int out_bit_period; /* How often to do an alternate frame size. */
+    unsigned int out_bit_period; /* How often to do an alternate frame size. */
     uint64_t out_bit_time; /* Time in nsec for a bitsize to be sent. */
 
     unsigned int debug;
+    bool check_ax25;
+    bool do_crc;
     gensiods framecount;
     gensiods framenr;
 
@@ -358,7 +373,9 @@ struct afskmdm_filter {
 	KEY_OPEN,
 	KEY_IN_CLOSE,
     } key_io_state;
+    enum afskmdm_keytype keytype;
     struct gensio *key_io;
+    struct sergensio *key_sio;
     char *key;
     char *keyon;
     char *keyoff;
@@ -551,16 +568,25 @@ afskmdm_ax25_prmsg(struct gensio_os_funcs *o,
 }
 
 static void
-afskmdm_print_msg(struct afskmdm_filter *sfilter, unsigned int msgn,
+afskmdm_print_msg(struct afskmdm_filter *sfilter, char *t, unsigned int msgn,
 		  unsigned char *buf, unsigned int buflen,
 		  bool pr_msgn)
 {
+    struct gensio_os_funcs *o = sfilter->o;
     struct gensio_fdump h;
 
+    if (sfilter->debug & 0x10) {
+	gensio_time time;
+
+	o->get_monotonic_time(o, &time);
+	printf("%lld:%6.6d: ",
+	       (long long) time.secs, (time.nsecs + 500) / 1000);
+    }
+
     if (pr_msgn) {
-	printf("MSG(%u %u):", msgn, buflen);
+	printf("%sMSG(%u %u):", t, msgn, buflen);
     } else {
-	printf("MSG(%u):", buflen);
+	printf("%sMSG(%u):", t, buflen);
 	afskmdm_ax25_prmsg(sfilter->o, buf, buflen);
     }
     printf("\n");
@@ -570,11 +596,94 @@ afskmdm_print_msg(struct afskmdm_filter *sfilter, unsigned int msgn,
     fflush(stdout);
 }
 
+static void
+keyop_done(struct sergensio *sio, int err, unsigned int val, void *cb_data)
+{
+    if (err)
+	gensio_log(cb_data, GENSIO_LOG_WARNING,
+		   "afskmdm: Error keying transmitter: %s\n",
+		   gensio_err_to_str(err));
+}
+
+static void
+afskmdm_do_keyon(struct afskmdm_filter *sfilter)
+{
+    if (!sfilter->key_io)
+	return;
+    switch (sfilter->keytype) {
+    case KEY_RW:
+	gensio_write(sfilter->key_io, NULL,
+		     sfilter->keyon, strlen(sfilter->keyon), NULL);
+	break;
+
+    case KEY_RTS:
+	sergensio_rts(sfilter->key_sio, SERGENSIO_RTS_ON, keyop_done,
+		      sfilter->o);
+	break;
+
+    case KEY_RTSINV:
+	sergensio_rts(sfilter->key_sio, SERGENSIO_RTS_OFF, keyop_done,
+		      sfilter->o);
+	break;
+
+    case KEY_DTR:
+	sergensio_dtr(sfilter->key_sio, SERGENSIO_DTR_ON, keyop_done,
+		      sfilter->o);
+	break;
+
+    case KEY_DTRINV:
+	sergensio_dtr(sfilter->key_sio, SERGENSIO_DTR_OFF, keyop_done,
+		      sfilter->o);
+	break;
+
+    case KEY_CM108: /* Should never happen. */
+	assert(0);
+    }
+    sfilter->keyed = true;
+}
+
+static void
+afskmdm_do_keyoff(struct afskmdm_filter *sfilter)
+{
+    if (!sfilter->key_io)
+	return;
+    switch (sfilter->keytype) {
+    case KEY_RW:
+	gensio_write(sfilter->key_io, NULL,
+		     sfilter->keyoff, strlen(sfilter->keyoff), NULL);
+	break;
+
+    case KEY_RTS:
+	sergensio_rts(sfilter->key_sio, SERGENSIO_RTS_OFF, keyop_done,
+		      sfilter->o);
+	break;
+
+    case KEY_RTSINV:
+	sergensio_rts(sfilter->key_sio, SERGENSIO_RTS_ON, keyop_done,
+		      sfilter->o);
+	break;
+
+    case KEY_DTR:
+	sergensio_dtr(sfilter->key_sio, SERGENSIO_DTR_OFF, keyop_done,
+		      sfilter->o);
+	break;
+
+    case KEY_DTRINV:
+	sergensio_dtr(sfilter->key_sio, SERGENSIO_DTR_ON, keyop_done,
+		      sfilter->o);
+	break;
+
+    case KEY_CM108: /* Should never happen. */
+	assert(0);
+    }
+    sfilter->keyed = false;
+}
+
 static int
 key_cb(struct gensio *io, void *user_data, int event, int err,
        unsigned char *buf, gensiods *buflen, const char *const *auxdata)
 {
-    //struct afskmdm_filter *sfilter = user_data;
+    struct afskmdm_filter *sfilter = user_data;
 
     switch(event) {
     case GENSIO_EVENT_READ:
@@ -583,6 +692,13 @@ key_cb(struct gensio *io, void *user_data, int event, int err,
     case GENSIO_EVENT_WRITE_READY:
 	return 0;
 
+    case GENSIO_EVENT_PARMLOG: {
+	struct gensio_parmlog_data *d = (struct gensio_parmlog_data *) buf;
+
+	gensio_pparm_vlog(&sfilter->p, d->log, d->args);
+	return 0;
+    }
+
     default:
 	return GE_NOTSUP;
     }
@@ -593,10 +709,15 @@ key_open_done(struct gensio *io, int err, void *open_data)
 {
     struct afskmdm_filter *sfilter = open_data;
 
-    if (err)
+    if (err) {
 	sfilter->key_io_state = KEY_CLOSED;
-    else
+	gensio_log(sfilter->o, GENSIO_LOG_ERR,
+		   "afskmdm: Error from open key I/O '%s': %s", sfilter->key,
+		   gensio_err_to_str(err));
+    } else {
 	sfilter->key_io_state = KEY_OPEN;
+	afskmdm_do_keyoff(sfilter);
+    }
     sfilter->key_err = err;
 
     /* Just turn on read and ignore what we get. */
@@ -623,8 +744,12 @@ afskmdm_try_connect(struct gensio_filter *filter, gensio_time *timeout,
 	sfilter->key_io_state != KEY_OPEN) {
 
 	err = gensio_open(sfilter->key_io, key_open_done, sfilter);
-	if (err)
+	if (err) {
+	    gensio_log(sfilter->o, GENSIO_LOG_ERR,
+		       "afskmdm: Unable to open key I/O '%s': %s", sfilter->key,
+		       gensio_err_to_str(err));
 	    return err;
+	}
 	sfilter->key_io_state = KEY_IN_OPEN;
     }
     if (sfilter->key_io_state == KEY_IN_OPEN) {
@@ -651,16 +776,16 @@ afskmdm_try_disconnect(struct gensio_filter *filter, gensio_time *timeout,
     int err;
 
     if (sfilter->key_io_state == KEY_OPEN) {
-	if (sfilter->keyed) {
-	    gensio_write(sfilter->key_io, NULL,
-			 sfilter->keyoff, strlen(sfilter->keyoff), NULL);
-	    sfilter->keyed = false;
-	}
+	afskmdm_do_keyoff(sfilter);
 	err = gensio_close(sfilter->key_io, key_close_done, sfilter);
-	if (err)
+	if (err) {
 	    sfilter->key_io_state = KEY_CLOSED;
-	else
+	    gensio_log(sfilter->o, GENSIO_LOG_WARNING,
+		       "afskmdm: Error from close key I/O '%s': %s",
+		       sfilter->key, gensio_err_to_str(err));
+	} else {
 	    sfilter->key_io_state = KEY_IN_CLOSE;
+	}
     }
     if (sfilter->key_io_state == KEY_IN_CLOSE) {
 	timeout->secs = 0;
@@ -683,11 +808,7 @@ afskmdm_start_xmit(struct afskmdm_filter *sfilter)
 			   sfilter->out_bit_time / 8);
     sfilter->bitstuff = false;
     sfilter->starting_output_ready = true;
-    if (sfilter->key_io) {
-	gensio_write(sfilter->key_io, NULL,
-		     sfilter->keyon, strlen(sfilter->keyon), NULL);
-	sfilter->keyed = true;
-    }
+    afskmdm_do_keyon(sfilter);
 }
 
 static void
@@ -731,11 +852,8 @@ afskmdm_timeout_done(struct gensio_filter *filter)
     } else {
 	sfilter->transmit_state = NOT_SENDING;
     }
-    if (sfilter->keyed) {
-	gensio_write(sfilter->key_io, NULL,
-		     sfilter->keyoff, strlen(sfilter->keyoff), NULL);
-	sfilter->keyed = false;
-    }
+    if (sfilter->keyed)
+	afskmdm_do_keyoff(sfilter);
     afskmdm_unlock(sfilter);
 
     return 0;
@@ -991,19 +1109,20 @@ afskmdm_ul_write(struct gensio_filter *filter,
 	goto out_process;
 
     if (sfilter->debug & 8) {
-	printf("W");
-	afskmdm_print_msg(sfilter, 0, sfilter->wrbufs[cbuf].data,
+	afskmdm_print_msg(sfilter, "W", 0, sfilter->wrbufs[cbuf].data,
 			  sfilter->wrbufs[cbuf].len, false);
     }
 
-    /* We have two extra bytes on the end for the CRC, no check needed. */
-    crc = 0xffff;
-    len = sfilter->wrbufs[cbuf].len;
-    crc16_ccitt(sfilter->wrbufs[cbuf].data, len, &crc);
-    crc ^= 0xffff;
-    sfilter->wrbufs[cbuf].data[len++] = crc & 0xff;
-    sfilter->wrbufs[cbuf].data[len++] = (crc >> 8) & 0xff;
-    sfilter->wrbufs[cbuf].len = len;
+    if (sfilter->do_crc) {
+	/* We have two extra bytes on the end for the CRC, no check needed. */
+	crc = 0xffff;
+	len = sfilter->wrbufs[cbuf].len;
+	crc16_ccitt(sfilter->wrbufs[cbuf].data, len, &crc);
+	crc ^= 0xffff;
+	sfilter->wrbufs[cbuf].data[len++] = crc & 0xff;
+	sfilter->wrbufs[cbuf].data[len++] = (crc >> 8) & 0xff;
+	sfilter->wrbufs[cbuf].len = len;
+    }
 
     sfilter->nr_wrbufs++;
     if (sfilter->transmit_state != NOT_SENDING)
@@ -1145,19 +1264,6 @@ afskmdm_convolve(struct afskmdm_filter *sfilter, float *convdata,
     }
 }
 
-static void
-afskmdm_handle_new_byte(struct afskmdm_filter *sfilter, unsigned int msgn,
-			struct wmsg *w)
-{
-    if (sfilter->debug & 2)
-	printf("BYTE(%d): %2.2x\n", msgn, w->curr_byte);
-    if (w->read_data_len < sfilter->max_read_size)
-	w->read_data[w->read_data_len] = w->curr_byte;
-    w->curr_byte = 0;
-    w->curr_bit_pos = 0;
-    w->read_data_len++;
-}
-
 static void
 afskmdm_drop_wmsg(struct afskmdm_filter *sfilter, unsigned int wset,
 		  unsigned int msgn, struct wmsg *w, bool at_flag)
@@ -1192,6 +1298,23 @@ afskmdm_drop_wmsg(struct afskmdm_filter *sfilter, unsigned int wset,
     }
 }
 
+static void
+afskmdm_handle_new_byte(struct afskmdm_filter *sfilter,
+			unsigned int wset, unsigned int msgn,
+			struct wmsg *w)
+{
+    if (sfilter->debug & 2)
+	printf("BYTE(%d): %2.2x\n", msgn, w->curr_byte);
+    if (w->read_data_len >= sfilter->max_read_size) {
+	afskmdm_drop_wmsg(sfilter, wset, msgn, w, false);
+	return;
+    }
+    w->read_data[w->read_data_len] = w->curr_byte;
+    w->curr_byte = 0;
+    w->curr_bit_pos = 0;
+    w->read_data_len++;
+}
+
 static void
 afskmdm_handle_new_message(struct afskmdm_filter *sfilter, unsigned int pos,
 			   unsigned int wset, unsigned int msgn, struct wmsg *w)
@@ -1203,7 +1326,8 @@ afskmdm_handle_new_message(struct afskmdm_filter *sfilter, unsigned int pos,
 	goto bad_msg;
 
     if (sfilter->debug & 1) {
-	afskmdm_print_msg(sfilter, msgn, w->read_data, w->read_data_len, true);
+	afskmdm_print_msg(sfilter, "", msgn, w->read_data, w->read_data_len,
+			  true);
 	printf("    bitpos %d  endframe %lu\n", w->curr_bit_pos,
 	       sfilter->framenr + pos - sfilter->prevread_size);
     }
@@ -1216,27 +1340,43 @@ afskmdm_handle_new_message(struct afskmdm_filter *sfilter, unsigned int pos,
     if (w->curr_bit_pos != 6 && w->curr_bit_pos != 5)
 	goto bad_msg;
 
-    crc = 0xffff;
-    crc16_ccitt(w->read_data, w->read_data_len - 2, &crc);
-    crc ^= 0xffff;
-    msgcrc = ((w->read_data[w->read_data_len - 1] << 8) |
-	      w->read_data[w->read_data_len - 2]);
+    if (sfilter->do_crc) {
+	crc = 0xffff;
+	crc16_ccitt(w->read_data, w->read_data_len - 2, &crc);
+	crc ^= 0xffff;
+	msgcrc = ((w->read_data[w->read_data_len - 1] << 8) |
+		  w->read_data[w->read_data_len - 2]);
 
-    if (sfilter->debug & 1)
-	printf("    CRC %4.4x, MSGCRC %4.4x\n", crc, msgcrc);
+	if (sfilter->debug & 1)
+	    printf("    CRC %4.4x, MSGCRC %4.4x\n", crc, msgcrc);
 
-    if (crc != msgcrc)
-	goto bad_msg;
+	if (crc != msgcrc)
+	    goto bad_msg;
 
-    w->read_data_len -= 2;
+	w->read_data_len -= 2;
 
-    if (sfilter->debug & 8) {
-	printf("R");
-	afskmdm_print_msg(sfilter, 0, w->read_data, w->read_data_len, false);
+	if (sfilter->check_ax25) {
+	    struct gensio_ax25_addr iaddr;
+	    gensiods ipos;
+	    int err;
+
+	    if (w->read_data_len < 16)
+		goto bad_msg;
+
+	    ipos = 0;
+	    err = decode_ax25_addr(sfilter->o, w->read_data, &ipos,
+				   w->read_data_len, 0, &iaddr);
+	    if (err)
+		goto bad_msg;
+	}
     }
 
-    if (w->read_data_len <= sfilter->max_read_size &&
-		sfilter->deliver_data_len == 0) {
+    if (sfilter->debug & 8) {
+	afskmdm_print_msg(sfilter, "R", 0, w->read_data, w->read_data_len,
+			  false);
+    }
+
+    if (sfilter->deliver_data_len == 0) {
 	unsigned char *tmp;
 
 	tmp = w->read_data;
@@ -1407,7 +1547,7 @@ afskmdm_process_bit(struct afskmdm_filter *sfilter, unsigned int pos,
 
 	w->curr_byte |= bit << w->curr_bit_pos;
 	if (w->curr_bit_pos == 7)
-	    afskmdm_handle_new_byte(sfilter, msgn, w);
+	    afskmdm_handle_new_byte(sfilter, wset, msgn, w);
 	else
 	    w->curr_bit_pos++;
 	break;
@@ -1462,10 +1602,8 @@ process_powers(struct afskmdm_filter *sfilter,
 }
 
 /*
- * Do a convolution at 1200Hz and 2200Hz over the data.  Return
- * whether the power at 1200Hz (bit=0) or 2200Hz (bit=1) is greater.
- * Returns the certainty of the bit, which is the ratio of the two
- * power levels.
+ * Do a convolution at mark and space the data then call the bit
+ * processing with the info extracted from the data.
  */
 static void
 afskmdm_check_for_data(struct afskmdm_filter *sfilter, unsigned int *curpos,
@@ -1654,13 +1792,13 @@ afskmdm_calc_fir_coefs(struct gensio_os_funcs *o,
 		       unsigned int *rn)
 {
     double tba = transband / samplerate;
-    double w = 2 * M_PI * cutoff / samplerate;
+    double coa = cutoff / samplerate;
+    double w = 2 * M_PI * (coa + .5 * tba);
     unsigned int i;
     /* For a hamming filter, transition band ~ (3.3 / N). */
     unsigned int n = 3.3 / tba / 2;
     double N = n * 2 + 1;
-    double dx = (n * 2) / (n * 2 - 1);
-    double x = dx;
+    double x = 1.0;
     float *h;
 
     h = o->zalloc(o, n * sizeof(float));
@@ -1671,7 +1809,7 @@ afskmdm_calc_fir_coefs(struct gensio_os_funcs *o,
 	double tmp;
 
 	/* h(x) = 2 * f * sinc() */
-	tmp = sin(w * x) / (M_PI * x);
+	tmp = sin(x * w) / (x * M_PI);
 
 	/* Hamming window */
 	tmp *= .54 - .46 * cos(2 * M_PI * (i + 1) / N);
@@ -1680,7 +1818,7 @@ afskmdm_calc_fir_coefs(struct gensio_os_funcs *o,
 
 	if (i == 0)
 	    break;
-	x += dx;
+	x += 1.0;
     }
     *rn = n;
     return h;
@@ -2132,6 +2270,8 @@ struct gensio_afskmdm_data {
     float space_freq;
     unsigned int data_rate;
     unsigned int debug;
+    bool check_ax25;
+    bool do_crc;
     unsigned int in_framerate;
     unsigned int out_framerate;
     unsigned int in_chunksize;
@@ -2153,6 +2293,8 @@ struct gensio_afskmdm_data {
     unsigned int tx_predelay_time;
     float volume;
     const char *key;
+    int keytype;
+    unsigned int keybit;
     const char *keyon;
     const char *keyoff;
     bool full_duplex;
@@ -2167,7 +2309,7 @@ afskmdm_setup_transmit(struct afskmdm_filter *sfilter,
     unsigned int i;
     struct xmit_entry *e;
 
-    sfilter->mark_xmit_len = data->out_framerate / data->mark_freq * 2 + 1;
+    sfilter->mark_xmit_len = data->out_framerate / data->mark_freq * 2 + 2;
     if (sfilter->mark_xmit_len < 2 * sfilter->out_bitsize + 1)
 	sfilter->mark_xmit_len = 2 * sfilter->out_bitsize + 1;
     sfilter->mark_xmit = o->zalloc(o, sizeof(float) * sfilter->mark_xmit_len);
@@ -2178,7 +2320,7 @@ afskmdm_setup_transmit(struct afskmdm_filter *sfilter,
 	sfilter->mark_xmit[i] = sin(v / fbitsize) * data->volume;
     }
 
-    sfilter->space_xmit_len = data->out_framerate / data->space_freq * 2 + 1;
+    sfilter->space_xmit_len = data->out_framerate / data->space_freq * 2 + 2;
     if (sfilter->space_xmit_len < 2 * sfilter->out_bitsize + 1)
 	sfilter->space_xmit_len = 2 * sfilter->out_bitsize + 1;
     sfilter->space_xmit = o->zalloc(o, sizeof(float) * sfilter->space_xmit_len);
@@ -2204,7 +2346,9 @@ afskmdm_setup_transmit(struct afskmdm_filter *sfilter,
 }
 
 static struct gensio_filter *
-gensio_afskmdm_filter_raw_alloc(struct gensio_os_funcs *o,
+gensio_afskmdm_filter_raw_alloc(struct gensio_pparm_info *p,
+				struct gensio_os_funcs *o,
+				struct gensio *child,
 				struct gensio_afskmdm_data *data)
 {
     struct afskmdm_filter *sfilter;
@@ -2225,6 +2369,8 @@ gensio_afskmdm_filter_raw_alloc(struct gensio_os_funcs *o,
     sfilter->max_write_size = data->max_write_size;
     sfilter->max_read_size = data->max_read_size + 2; /* Extra 2 for the CRC. */
     sfilter->debug = data->debug;
+    sfilter->check_ax25 = data->check_ax25;
+    sfilter->do_crc = data->do_crc;
     sfilter->prev_xmit_level = -1;
     sfilter->prev_recv_level = 0;
     sfilter->in_chunksize = data->in_chunksize;
@@ -2241,6 +2387,7 @@ gensio_afskmdm_filter_raw_alloc(struct gensio_os_funcs *o,
 	if (!sfilter->key)
 	    goto out_nomem;
     }
+    sfilter->keytype = data->keytype;
     if (data->keyon) {
 	sfilter->keyon = gensio_strdup(o, data->keyon);
 	if (!sfilter->keyon)
@@ -2401,8 +2548,12 @@ gensio_afskmdm_filter_raw_alloc(struct gensio_os_funcs *o,
 	goto out_nomem;
 
     for (i = 0; i < NR_WRITE_BUFS; i++) {
-	/* Add 2 to allow for the CRC to be added. */
-	sfilter->wrbufs[i].data = o->zalloc(o, sfilter->max_write_size + 2);
+	gensiods wrsz = sfilter->max_write_size;
+
+	if (sfilter->do_crc)
+	    /* Add 2 to allow for the CRC to be added. */
+	    wrsz += 2;
+	sfilter->wrbufs[i].data = o->zalloc(o, wrsz);
 	if (!sfilter->wrbufs[i].data)
 	    goto out_nomem;
     }
@@ -2423,11 +2574,50 @@ gensio_afskmdm_filter_raw_alloc(struct gensio_os_funcs *o,
     if (afskmdm_setup_transmit(sfilter, data, fbitsize))
 	goto out_nomem;
 
+    sfilter->p = *p;
+    if (sfilter->keytype == KEY_CM108) {
+	char name[100];
+	gensiods len = sizeof(name);
+	int err;
+
+	strcpy(name, "out");
+	err = gensio_control(child, 0, true, GENSIO_CONTROL_LADDR, name, &len);
+	if (err) {
+	    gensio_pparm_log(p, "Unable to get the output sound card name for"
+			     " fetching the cm108 parameter: %s.",
+			     gensio_err_to_str(err));
+	    goto out_nomem;
+	}
+	if (sfilter->key)
+	    o->free(o, sfilter->key);
+	sfilter->keytype = KEY_RW;
+	sfilter->key = gensio_alloc_sprintf(o, "cm108gpio(bit=%u),%s",
+					    data->keybit, name);
+	if (!sfilter->key)
+	    goto out_nomem;
+    }
     if (sfilter->key) {
 	int err = str_to_gensio(sfilter->key, o, key_cb, sfilter,
 				&sfilter->key_io);
-	if (err)
+	if (err) {
+	    gensio_pparm_log(p, "Could not allocate key gensio '%s': %s",
+			     sfilter->key, gensio_err_to_str(err));
 	    goto out_nomem;
+	}
+	switch (sfilter->keytype) {
+	case KEY_RTS: case KEY_RTSINV: case KEY_DTR: case KEY_DTRINV:
+	    sfilter->key_sio = gensio_to_sergensio(sfilter->key_io);
+	    if (!sfilter->key_sio) {
+		gensio_pparm_log(p, "A serial keytype was given, '%s',"
+				 " but it is not a serial gensio",
+				 sfilter->key);
+		goto out_nomem;
+	    }
+	    break;
+
+	default:
+	    break;
+	}
     }
 
     return sfilter->filter;
@@ -2460,8 +2650,19 @@ static struct gensio_enum_val filttype_enums[] = {
     { }
 };
 
+static struct gensio_enum_val keytype_enums[] = {
+    { .name = "rw", .val = KEY_RW },
+    { .name = "rts", .val = KEY_RTS },
+    { .name = "rtsinv", .val = KEY_RTSINV },
+    { .name = "dtr", .val = KEY_DTR },
+    { .name = "dtrinv", .val = KEY_DTRINV },
+    { .name = "cm108", .val = KEY_CM108 },
+    { }
+};
+
 int
-gensio_afskmdm_filter_alloc(struct gensio_os_funcs *o,
+gensio_afskmdm_filter_alloc(struct gensio_pparm_info *p,
+			    struct gensio_os_funcs *o,
 			    struct gensio *child,
 			    const char * const args[],
 			    struct gensio_filter **rfilter)
@@ -2493,6 +2694,11 @@ gensio_afskmdm_filter_alloc(struct gensio_os_funcs *o,
 	.tx_predelay_time = 500,
 	.volume = .75,
 	.full_duplex = false,
+	.keytype = KEY_RW,
+	.keybit = 3,
+	.keyon = "T 1\n",
+	.keyoff = "T 0\n",
+	.do_crc = true
     };
     unsigned int i;
     int err;
@@ -2503,13 +2709,19 @@ gensio_afskmdm_filter_alloc(struct gensio_os_funcs *o,
 
     err = afskmdm_child_getuint(child, GENSIO_CONTROL_IN_BUFSIZE,
 				&data.in_chunksize);
-    if (err)
+    if (err) {
+	gensio_pparm_slog(p, "Unable to get child input buffer size,"
+			  " is it a sound device?");
 	return GE_INCONSISTENT;
+    }
 
     err = afskmdm_child_getuint(child, GENSIO_CONTROL_OUT_BUFSIZE,
 				&data.out_chunksize);
-    if (err)
+    if (err) {
+	gensio_pparm_slog(p, "Unable to get child output buffer size,"
+			  " is it a sound device?");
 	return GE_INCONSISTENT;
+    }
 
     /* Don't care if these fail, we will check later. */
     afskmdm_child_getuint(child, GENSIO_CONTROL_IN_RATE,
@@ -2524,101 +2736,131 @@ gensio_afskmdm_filter_alloc(struct gensio_os_funcs *o,
     cdata_len = sizeof(cdata);
     err = gensio_control(child, GENSIO_CONTROL_DEPTH_FIRST, true,
 			 GENSIO_CONTROL_IN_FORMAT, cdata, &cdata_len);
-    if (err || strcmp(cdata, "float") != 0)
+    if (err) {
+	gensio_pparm_slog(p, "Unable to get child input format,"
+			  " is it a sound device?");
 	return GE_INCONSISTENT;
+    }
+    if (strcmp(cdata, "float") != 0) {
+	gensio_pparm_slog(p, "Child input format is not float");
+	return GE_INCONSISTENT;
+    }
 
     cdata_len = sizeof(cdata);
     err = gensio_control(child, GENSIO_CONTROL_DEPTH_FIRST, true,
 			 GENSIO_CONTROL_OUT_FORMAT, cdata, &cdata_len);
-    if (err || strcmp(cdata, "float") != 0)
+    if (err) {
+	gensio_pparm_slog(p, "Unable to get child output format,"
+			  " is it a sound device?");
 	return GE_INCONSISTENT;
+    }
+    if (strcmp(cdata, "float") != 0) {
+	gensio_pparm_slog(p, "Child output format is not float");
+	return GE_INCONSISTENT;
+    }
 
     for (i = 0; args && args[i]; i++) {
-	if (gensio_check_keyds(args[i], "readbuf", &data.max_read_size) > 0)
+	if (gensio_pparm_ds(p, args[i], "readbuf", &data.max_read_size) > 0)
 	    continue;
-	if (gensio_check_keyds(args[i], "writebuf", &data.max_write_size) > 0)
+	if (gensio_pparm_ds(p, args[i], "writebuf", &data.max_write_size) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "nchans", &data.in_nchans) > 0) {
+	if (gensio_pparm_uint(p, args[i], "nchans", &data.in_nchans) > 0) {
 	    data.out_nchans = data.in_nchans;
 	    continue;
 	}
-	if (gensio_check_keyuint(args[i], "in_nchans", &data.in_nchans) > 0)
+	if (gensio_pparm_uint(p, args[i], "in_nchans", &data.in_nchans) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "out_nchans", &data.out_nchans) > 0)
+	if (gensio_pparm_uint(p, args[i], "out_nchans", &data.out_nchans) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "chan", &data.in_chan) > 0) {
+	if (gensio_pparm_uint(p, args[i], "chan", &data.in_chan) > 0) {
 	    data.out_chans = 1 << data.in_chan;
 	    continue;
 	}
-	if (gensio_check_keyuint(args[i], "in_chan", &data.in_chan) > 0)
+	if (gensio_pparm_uint(p, args[i], "in_chan", &data.in_chan) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "out_chans", &data.out_chans) > 0)
+	if (gensio_pparm_uint(p, args[i], "out_chans", &data.out_chans) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "out_chan", &chan) > 0) {
+	if (gensio_pparm_uint(p, args[i], "out_chan", &chan) > 0) {
 	    data.out_chans = 1 << chan;
 	    continue;
 	}
-	if (gensio_check_keyuint(args[i], "samplerate",
+	if (gensio_pparm_uint(p, args[i], "samplerate",
 				 &data.in_framerate) > 0) {
 	    data.out_framerate = data.in_framerate;
 	    continue;
 	}
-	if (gensio_check_keyuint(args[i], "in_samplerate",
+	if (gensio_pparm_uint(p, args[i], "in_samplerate",
 				 &data.in_framerate) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "out_samplerate",
+	if (gensio_pparm_uint(p, args[i], "out_samplerate",
 				 &data.in_framerate) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "wmsgs", &data.max_wmsgs) > 0)
+	if (gensio_pparm_uint(p, args[i], "wmsgs", &data.max_wmsgs) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "wmsg-extra", &wmsg_extra) > 0)
+	if (gensio_pparm_uint(p, args[i], "wmsg-extra", &wmsg_extra) > 0)
 	    continue;
-	if (gensio_check_keyfloat(args[i], "min-certainty",
+	if (gensio_pparm_float(p, args[i], "min-certainty",
 				  &data.min_certainty) > 0)
 	    continue;
-	if (gensio_check_keyenum(args[i], "filttype", filttype_enums,
+	if (gensio_pparm_enum(p, args[i], "filttype", filttype_enums,
 				 &data.filt_type) > 0) {
 	    data.filt_type_set = true;
 	    continue;
 	}
-	if (gensio_check_keyuint(args[i], "lpcutoff", &data.lpcutoff) > 0)
+	if (gensio_pparm_uint(p, args[i], "lpcutoff", &data.lpcutoff) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "trfreq", &data.transition_freq) > 0)
+	if (gensio_pparm_uint(p, args[i], "trfreq", &data.transition_freq) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "tx-preamble",
+	if (gensio_pparm_uint(p, args[i], "tx-preamble",
 				 &data.tx_preamble_time) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "tx-tail",
+	if (gensio_pparm_uint(p, args[i], "tx-tail",
 				 &data.tx_postamble_time) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "tx-predelay",
+	if (gensio_pparm_uint(p, args[i], "tx-predelay",
 				 &data.tx_predelay_time) > 0)
 	    continue;
-	if (gensio_check_keyfloat(args[i], "volume", &data.volume) > 0)
+	if (gensio_pparm_float(p, args[i], "volume", &data.volume) > 0)
 	    continue;
-	if (gensio_check_keyvalue(args[i], "key", &data.key) > 0)
+	if (gensio_pparm_value(p, args[i], "key", &data.key) > 0)
 	    continue;
-	if (gensio_check_keyvalue(args[i], "keyon", &data.keyon) > 0)
+	if (gensio_pparm_enum(p, args[i], "keytype", keytype_enums,
+			      &data.keytype) > 0)
 	    continue;
-	if (gensio_check_keyvalue(args[i], "keyoff", &data.keyoff) > 0)
+	if (gensio_pparm_uint(p, args[i], "keybit", &data.keybit) > 0)
 	    continue;
-	if (gensio_check_keybool(args[i], "full-duplex", &data.full_duplex) > 0)
+	if (gensio_pparm_value(p, args[i], "keyon", &data.keyon) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "debug", &data.debug) > 0)
+	if (gensio_pparm_value(p, args[i], "keyoff", &data.keyoff) > 0)
 	    continue;
+	if (gensio_pparm_bool(p, args[i], "full-duplex", &data.full_duplex) > 0)
+	    continue;
+	if (gensio_pparm_uint(p, args[i], "debug", &data.debug) > 0)
+	    continue;
+	if (gensio_pparm_bool(p, args[i], "checkax25", &data.check_ax25) > 0)
+	    continue;
+	if (gensio_pparm_bool(p, args[i], "crc", &data.do_crc) > 0)
+	    continue;
+	gensio_pparm_unknown_parm(p, args[i]);
 	return GE_INVAL;
     }
 
-    if (data.in_framerate == 0 || data.out_framerate == 0 ||
-		data.in_chunksize == 0 || data.out_chunksize == 0 ||
-		data.in_nchans == 0 || data.out_nchans == 0 ||
-		data.in_chan >= data.in_nchans ||
-		data.out_chans >= (1 << data.out_nchans) ||
-		data.max_wmsgs == 0)
-	return GE_INVAL;
+#define MY_STRINGIZE(s) #s
+#define CHECK_VAL(d, cmp, v)						\
+    if (data.d cmp v) {							\
+	gensio_pparm_log(p, #d " cannot be " #cmp " %d\n", v);		\
+	return GE_INVAL;						\
+    }
 
-    if (data.key && (!data.keyon || !data.keyoff))
-	return GE_INVAL;
+    CHECK_VAL(in_framerate, ==, 0);
+    CHECK_VAL(out_framerate, ==, 0);
+    CHECK_VAL(in_chunksize, ==, 0);
+    CHECK_VAL(out_chunksize, ==, 0);
+    CHECK_VAL(in_nchans, ==, 0);
+    CHECK_VAL(out_nchans, ==, 0);
+    CHECK_VAL(in_chan, >=, data.in_nchans);
+    CHECK_VAL(out_chans, >=, (1U << data.out_nchans))
+    CHECK_VAL(max_wmsgs, ==, 0);
 
     /*
      * For lower sample rates a FIR filter doesn't use as much CPU and
@@ -2634,7 +2876,7 @@ gensio_afskmdm_filter_alloc(struct gensio_os_funcs *o,
 
     data.wmsg_sets = wmsg_extra * 2 + 1;
 
-    filter = gensio_afskmdm_filter_raw_alloc(o, &data);
+    filter = gensio_afskmdm_filter_raw_alloc(p, o, child, &data);
     if (!filter)
 	return GE_NOMEM;
 
