@@ -9,6 +9,7 @@
 
 #include "gensio_filter_certauth.h"
 #include <gensio/gensio_err.h>
+#include <gensio/gensio_time.h>
 
 #ifdef _WIN32
 #define DIRSEP '\\'
@@ -32,6 +33,9 @@ struct gensio_certauth_filter_data {
     bool enable_password;
     bool do_2fa; /* Ask for two-factor authentication, version 2+ */
 
+    /* Amount of time in which the connection process must complete. */
+    gensio_time con_timeout;
+
     /*
      * The following is only used for testing. so certauth can be run
      * over stdio for fuzz testing.  Do not document.
@@ -204,7 +208,7 @@ static void EVP_MD_CTX_free(EVP_MD_CTX *c)
 
 /*
  * State machines for both the client and the server, also message
- * numbers (except for CLIENT_START).
+ * numbers (for values < 100).
  */
 enum certauth_state {
     /*
@@ -244,6 +248,16 @@ enum certauth_state {
      */
     CERTAUTH_SERVERHELLO = 2,
 
+    /*
+     * Client waits for a random amount of time before processing the
+     * random challenge from the server.  This is to help with timing
+     * attacks.
+     *
+     * This will be transitioned from with a timer and will process
+     * the challenge and go to PASSWORD_REQUEST.
+     */
+    CERTAUTH_CLIENTDELAY = 110,
+
     /*
      * Server receives the challenge response verifies the reponse and
      * the certificate against the CA.
@@ -467,6 +481,15 @@ struct certauth_filter {
     /* Enable 2-factor authentication. */
     bool do_2fa;
 
+    /* try_connect() has been called at least once. */
+    bool started;
+
+    /* Time to wait for the connection to complete. */
+    gensio_time con_timeout;
+
+    /* Absolute time when the connection will time out. */
+    gensio_time contime_done;
+
     char *username;
     size_t username_len;
 
@@ -597,7 +620,7 @@ certauth_unlock(struct certauth_filter *sfilter)
 
 static void
 certauth_set_callbacks(struct gensio_filter *filter,
-		  gensio_filter_cb cb, void *cb_data)
+		       gensio_filter_cb cb, void *cb_data)
 {
     /* We don't currently use callbacks. */
 }
@@ -846,7 +869,8 @@ v3_certauth_add_challenge_rsp(struct certauth_filter *sfilter)
     certauth_write_byte(sfilter, CERTAUTH_CHALLENGE_RSP);
     lenpos = sfilter->write_buf_len;
     sfilter->write_buf_len += 2;
-    if (certauth_writeleft(sfilter) < EVP_PKEY_size(sfilter->pkey)) {
+    /* EVP_PKEY_size() docs say it always returns a positive number. */
+    if (certauth_writeleft(sfilter) < (gensiods) EVP_PKEY_size(sfilter->pkey)) {
 	gca_log_err(sfilter, "Key too large to fit in the data");
 	return GE_TOOBIG;
     }
@@ -1115,6 +1139,15 @@ certauth_send_server_done(struct certauth_filter *sfilter)
     certauth_write_byte(sfilter, CERTAUTH_END);
 }
 
+static void
+certauth_start_con_timeout(struct certauth_filter *sfilter)
+{
+    struct gensio_os_funcs *o = sfilter->o;
+
+    o->get_monotonic_time(o, &sfilter->contime_done);
+    gensio_time_add(&sfilter->contime_done, &sfilter->con_timeout);
+}
+
 static void
 set_digest(struct certauth_filter *sfilter)
 {
@@ -1125,7 +1158,8 @@ set_digest(struct certauth_filter *sfilter)
 }
 
 static int
-certauth_try_connect(struct gensio_filter *filter, gensio_time *timeout)
+certauth_try_connect(struct gensio_filter *filter, gensio_time *timeout,
+		     bool was_timeout)
 {
     struct certauth_filter *sfilter = filter_to_certauth(filter);
     struct gensio *io;
@@ -1133,12 +1167,27 @@ certauth_try_connect(struct gensio_filter *filter, gensio_time *timeout)
     bool password_requested = false;
     int err, rv;
     unsigned int req;
+    int64_t timeout_ns;
+    gensio_time time_now;
+    struct gensio_os_funcs *o = sfilter->o;
 
     certauth_lock(sfilter);
+    if (!sfilter->started) {
+	certauth_start_con_timeout(sfilter);
+	sfilter->started = true;
+    }
+
     if (sfilter->pending_err)
 	goto out_finish;
-    if (!sfilter->got_msg)
-	goto out_inprogress;
+    if (!sfilter->got_msg && !was_timeout)
+	goto out_inprogress2;
+
+    if (was_timeout && sfilter->state != CERTAUTH_CLIENTDELAY) {
+	gca_log_err(sfilter,
+		    "Remote system didn't finish connect in time");
+	sfilter->pending_err = GE_TIMEDOUT;
+	goto finish_result;
+    }
 
     switch (sfilter->state) {
     case CERTAUTH_CLIENT_START:
@@ -1256,6 +1305,39 @@ certauth_try_connect(struct gensio_filter *filter, gensio_time *timeout)
 	    sfilter->pending_err = GE_DATAMISSING;
 	    break;
 	}
+
+	/*
+	 * Start a random timer from 0 to 10ms to obscure the
+	 * processing time for avoiding timing attacks.
+	 */
+	if (!RAND_bytes((unsigned char *) &timeout_ns, sizeof(timeout_ns))) {
+	    gca_log_err(sfilter,
+			"Unable to get random response delay");
+	    sfilter->pending_err = GE_DATAMISSING;
+	    break;
+	}
+	if (timeout_ns < 0)
+	    timeout_ns = -timeout_ns;
+	timeout_ns %= GENSIO_MSECS_TO_NSECS(10);
+	timeout->secs = timeout_ns / GENSIO_NSECS_IN_SEC;
+	timeout->nsecs = timeout_ns % GENSIO_NSECS_IN_SEC;
+
+	sfilter->got_msg = false;
+	sfilter->state = CERTAUTH_CLIENTDELAY;
+	rv = GE_RETRY;
+	goto out;
+
+    case CERTAUTH_CLIENTDELAY:
+	if (!was_timeout) {
+	    if (sfilter->got_msg) {
+		gca_log_err(sfilter,
+		    "Got server data while client waiting in challenge delay");
+		sfilter->pending_err = GE_NOTREADY;
+		break;
+	    }
+	    goto out;
+	}
+
 	set_digest(sfilter);
 	sfilter->write_buf_len = 0;
 	certauth_write_byte(sfilter, CERTAUTH_CHALLENGE_RESPONSE);
@@ -1581,9 +1663,9 @@ certauth_try_connect(struct gensio_filter *filter, gensio_time *timeout)
 	sfilter->state = CERTAUTH_WAIT_WRITE_DONE;
 	/*
 	 * Leave got_msg enabled so we won't read any more until we go
-	 * to passthrough state.
+	 * to passthrough state.  Don't start the timer in this state.
 	 */
-	goto out_inprogress;
+	goto out_inprogress2;
 
     case CERTAUTH_SERVERDONE:
 	if (!sfilter->result) {
@@ -1615,6 +1697,15 @@ certauth_try_connect(struct gensio_filter *filter, gensio_time *timeout)
 
     sfilter->got_msg = false;
  out_inprogress:
+    o->get_monotonic_time(o, &time_now);
+    timeout_ns = gensio_time_diff_nsecs(&sfilter->contime_done, &time_now);
+    if (timeout_ns < 0)
+	timeout_ns = 0;
+    timeout->secs = timeout_ns / GENSIO_NSECS_IN_SEC;
+    timeout->nsecs = timeout_ns % GENSIO_NSECS_IN_SEC;
+    rv = GE_RETRY;
+    goto out;
+ out_inprogress2:
     rv = GE_INPROGRESS;
     goto out;
  out_finish:
@@ -2404,7 +2495,7 @@ int gensio_certauth_filter_func(struct gensio_filter *filter, int op,
 	return certauth_check_open_done(filter, data);
 
     case GENSIO_FILTER_FUNC_TRY_CONNECT:
-	return certauth_try_connect(filter, data);
+	return certauth_try_connect(filter, data, buflen);
 
     case GENSIO_FILTER_FUNC_TRY_DISCONNECT:
 	return certauth_try_disconnect(filter, data);
@@ -2428,7 +2519,7 @@ int gensio_certauth_filter_func(struct gensio_filter *filter, int op,
 
     case GENSIO_FILTER_FUNC_CONTROL:
 	return certauth_filter_control(filter, *((bool *) cbuf), buflen, data,
-				  count);
+				       count);
 
     case GENSIO_FILTER_FUNC_TIMEOUT:
     default:
@@ -2446,6 +2537,7 @@ gensio_certauth_filter_raw_alloc(struct gensio_os_funcs *o,
 				 const char *service,
 				 bool allow_authfail, bool use_child_auth,
 				 bool enable_password, bool do_2fa,
+				 gensio_time con_timeout,
 				 struct gensio_filter **rfilter)
 {
     struct certauth_filter *sfilter;
@@ -2461,6 +2553,7 @@ gensio_certauth_filter_raw_alloc(struct gensio_os_funcs *o,
     sfilter->use_child_auth = use_child_auth;
     sfilter->enable_password = enable_password;
     sfilter->do_2fa = do_2fa;
+    sfilter->con_timeout = con_timeout;
     sfilter->my_version = GENSIO_CERTAUTH_VERSION;
     sfilter->rsa_md5 = EVP_get_digestbyname("ssl3-md5");
     if (!sfilter->rsa_md5) {
@@ -2597,7 +2690,8 @@ gensio_certauth_filter_config_free(struct gensio_certauth_filter_data *data)
 }
 
 int
-gensio_certauth_filter_config(struct gensio_os_funcs *o,
+gensio_certauth_filter_config(struct gensio_pparm_info *p,
+			      struct gensio_os_funcs *o,
 			      const char * const args[],
 			      bool default_is_client,
 			      struct gensio_certauth_filter_data **rdata)
@@ -2651,39 +2745,47 @@ gensio_certauth_filter_config(struct gensio_os_funcs *o,
 	o->free(o, fstr);
     }
 
+    rv = gensio_get_default(o, "certauth", "con-timeout", false,
+			    GENSIO_DEFAULT_INT, NULL, &ival);
+    if (rv)
+	return rv;
+    data->con_timeout.secs = ival;
+    data->con_timeout.nsecs = 0;
+
+
     rv = GE_NOMEM;
     for (i = 0; args && args[i]; i++) {
-	if (gensio_check_keyvalue(args[i], "CA", &str) > 0) {
+	if (gensio_pparm_value(p, args[i], "CA", &str) > 0) {
 	    data->CAfilepath = gensio_strdup(o, str);
 	    if (!data->CAfilepath)
 		goto out_err;
 	    continue;
 	}
-	if (gensio_check_keyvalue(args[i], "key", &str) > 0) {
+	if (gensio_pparm_value(p, args[i], "key", &str) > 0) {
 	    data->keyfile = gensio_strdup(o, str);
 	    if (!data->keyfile)
 		goto out_err;
 	    continue;
 	}
-	if (gensio_check_keyvalue(args[i], "cert", &str) > 0) {
+	if (gensio_pparm_value(p, args[i], "cert", &str) > 0) {
 	    data->certfile = gensio_strdup(o, str);
 	    if (!data->certfile)
 		goto out_err;
 	    continue;
 	}
-	if (gensio_check_keyvalue(args[i], "username", &str) > 0) {
+	if (gensio_pparm_value(p, args[i], "username", &str) > 0) {
 	    data->username = gensio_strdup(o, str);
 	    if (!data->username)
 		goto out_err;
 	    continue;
 	}
-	if (gensio_check_keyvalue(args[i], "password", &str) > 0) {
+	if (gensio_pparm_value(p, args[i], "password", &str) > 0) {
 	    data->password = gensio_strdup(o, str);
 	    if (!data->password)
 		goto out_err;
 	    continue;
 	}
-	if (gensio_check_keyvalue(args[i], "2fa", &str) > 0) {
+	if (gensio_pparm_value(p, args[i], "2fa", &str) > 0) {
 	    data->len_2fa = strlen(str);
 	    if (data->len_2fa == 0)
 		goto out_err;
@@ -2692,30 +2794,34 @@ gensio_certauth_filter_config(struct gensio_os_funcs *o,
 		goto out_err;
 	    continue;
 	}
-	if (gensio_check_keyvalue(args[i], "service", &str) > 0) {
+	if (gensio_pparm_value(p, args[i], "service", &str) > 0) {
 	    data->service = gensio_strdup(o, str);
 	    if (!data->service)
 		goto out_err;
 	    continue;
 	}
-	if (gensio_check_keyboolv(args[i], "mode", "client", "server",
-				  &data->is_client) > 0)
+	if (gensio_pparm_boolv(p, args[i], "mode", "client", "server",
+			       &data->is_client) > 0)
 	    continue;
-	if (gensio_check_keybool(args[i], "allow-authfail",
-				 &data->allow_authfail) > 0)
+	if (gensio_pparm_bool(p, args[i], "allow-authfail",
+			      &data->allow_authfail) > 0)
 	    continue;
-	if (gensio_check_keybool(args[i], "use-child-auth",
-				 &data->use_child_auth) > 0)
+	if (gensio_pparm_bool(p, args[i], "use-child-auth",
+			      &data->use_child_auth) > 0)
 	    continue;
-	if (gensio_check_keybool(args[i], "enable-password",
-				 &data->enable_password) > 0)
+	if (gensio_pparm_bool(p, args[i], "enable-password",
+			      &data->enable_password) > 0)
 	    continue;
-	if (gensio_check_keybool(args[i], "enable-2fa",
-				 &data->do_2fa) > 0)
+	if (gensio_pparm_bool(p, args[i], "enable-2fa",
+			      &data->do_2fa) > 0)
 	    continue;
-	if (gensio_check_keybool(args[i], "allow-unencrypted",
-				 &data->allow_unencrypted) > 0)
+	if (gensio_pparm_time(p, args[i], "con-timeout", 's',
+			      &data->con_timeout) > 0)
 	    continue;
+	if (gensio_pparm_bool(p, args[i], "allow-unencrypted",
+			      &data->allow_unencrypted) > 0)
+	    continue;
+	gensio_pparm_unknown_parm(p, args[i]);
 	rv = GE_INVAL;
 	goto out_err;
     }
@@ -2790,12 +2896,29 @@ gensio_certauth_filter_config(struct gensio_os_funcs *o,
     }
 
     if (data->is_client) {
-	if (data->CAfilepath || data->do_2fa) {
+	if (data->CAfilepath) {
+	    gensio_pparm_slog(p, "CA is not valid for clients");
+	    rv = GE_INVAL;
+	    goto out_err;
+	}
+	if (data->do_2fa) {
+	    gensio_pparm_slog(p, "enable-2fa is not valid for clients");
 	    rv = GE_INVAL;
 	    goto out_err;
 	}
     } else {
-	if (data->keyfile || data->username || data->val_2fa) {
+	if (data->keyfile) {
+	    gensio_pparm_slog(p, "key is not valid for servers");
+	    rv = GE_INVAL;
+	    goto out_err;
+	}
+	if (data->username) {
+	    gensio_pparm_slog(p, "username is not valid for servers");
+	    rv = GE_INVAL;
+	    goto out_err;
+	}
+	if (data->val_2fa) {
+	    gensio_pparm_slog(p, "2fa is not valid for servers");
 	    rv = GE_INVAL;
 	    goto out_err;
 	}
@@ -2956,7 +3079,7 @@ gensio_certauth_filter_alloc(struct gensio_certauth_filter_data *data,
 					  data->allow_authfail,
 					  data->use_child_auth,
 					  data->enable_password,
-					  data->do_2fa,
+					  data->do_2fa, data->con_timeout,
 					  &filter);
     if (rv)
 	goto err;
