@@ -9,6 +9,7 @@
 #define GENSIO_FILTER_TELNET_H
 
 #include <gensio/gensio_base.h>
+#include <gensio/gensio_class.h>
 
 struct gensio_telnet_filter_callbacks {
     void (*got_sync)(void *handler_data);
@@ -16,6 +17,9 @@ struct gensio_telnet_filter_callbacks {
     int (*com_port_will_do)(void *handler_data, unsigned char cmd);
     void (*com_port_cmd)(void *handler_data, const unsigned char *option,
 			 unsigned int len);
+    int (*rfc1073_will_do)(void *handler_data, unsigned char cmd);
+    void (*rfc1073_cmd)(void *handler_data, const unsigned char *option,
+			unsigned int len);
     void (*timeout)(void *handler_data);
     void (*free)(void *handler_data);
 };
@@ -28,7 +32,8 @@ struct gensio_telnet_filter_rops {
     void (*start_timer)(struct gensio_filter *filter, gensio_time *timeout);
 };
 
-int gensio_telnet_filter_alloc(struct gensio_os_funcs *o,
+int gensio_telnet_filter_alloc(struct gensio_pparm_info *p,
+			       struct gensio_os_funcs *o,
 			       const char * const args[],
 			       bool default_is_client,
 			       const struct gensio_telnet_filter_callbacks *cbs,
