@@ -652,12 +652,13 @@ keepopen_gensio_alloc(struct gensio *child, const char * const args[],
     int i;
     struct gensio_time retry_time = { 1, 0 };
     bool discard_badwrites = false;
+    GENSIO_DECLARE_PPGENSIO(p, o, cb, "keepopen", user_data);
 
     for (i = 0; args && args[i]; i++) {
-	if (gensio_check_keytime(args[i], "retry-time", 'm', &retry_time) > 0)
+	if (gensio_pparm_time(&p, args[i], "retry-time", 'm', &retry_time) > 0)
 	    continue;
-	if (gensio_check_keybool(args[i], "discard-badwrites",
-				 &discard_badwrites) > 0)
+	if (gensio_pparm_bool(&p, args[i], "discard-badwrites",
+			      &discard_badwrites) > 0)
 	    continue;
 	return GE_INVAL;
     }
@@ -705,7 +706,8 @@ str_to_keepopen_gensio(const char *str, const char * const args[],
     int err;
     struct gensio *io2;
 
-    err = str_to_gensio(str, o, NULL, NULL, &io2);
+    /* cb is passed in for parmerr handling, it will be overriden later. */
+    err = str_to_gensio(str, o, cb, user_data, &io2);
     if (err)
 	return err;
 
