@@ -411,18 +411,29 @@ get_float(const unsigned char **in, unsigned int size, bool host_bswap)
     double v = 0;
 
     if (size == 4) {
-	float fv = *((float *) (*in));
+	char d[4];
+
+	memcpy(d, *in, 4);
 	if (host_bswap) {
-	    int32_t *iv = ((int32_t *) &fv);
-	    *iv = bswap_32(*((int32_t *) &fv));
+	    int32_t iv;
+
+	    memcpy(&iv, d, 4);
+	    iv = bswap_32(iv);
+	    memcpy(d, &iv, 4);
 	}
-	v = fv;
+	v = *((float *) d);
     } else if (size == 8) {
-	v = *((double *) (*in));
+	char d[8];
+
+	memcpy(d, *in, 8);
 	if (host_bswap) {
-	    int64_t *iv = ((int64_t *) &v);
-	    *iv = bswap_64(*((int64_t *) &v));
+	    int64_t iv;
+
+	    memcpy(&iv, d, 8);
+	    iv = bswap_64(iv);
+	    memcpy(d, &iv, 8);
 	}
+	v = *((double *) d);
     } else {
 	assert(0);
     }
@@ -539,7 +550,8 @@ conv_float_to_float_out(const unsigned char **in, unsigned char **out,
 
 struct sound_type {
     const char *name;
-    int (*setup)(struct sound_info *si, struct gensio_sound_info *io);
+    int (*setup)(struct gensio_pparm_info *p,
+		 struct sound_info *si, struct gensio_sound_info *io);
     void (*cleanup)(struct sound_info *si);
     int (*open_dev)(struct sound_info *si);
     void (*close_dev)(struct sound_info *si);
@@ -561,6 +573,7 @@ struct sound_info {
 
     struct sound_type *type;
     char *devname;
+    char *cardname;
     bool is_input;
 
     unsigned int samplerate; /* Frames per second. */
@@ -728,6 +741,10 @@ gensio_sound_ll_free(struct sound_ll *soundll)
 	o->free(o, soundll->in.devname);
     if (soundll->out.devname)
 	o->free(o, soundll->out.devname);
+    if (soundll->in.cardname)
+	o->free(o, soundll->in.cardname);
+    if (soundll->out.cardname)
+	o->free(o, soundll->out.cardname);
     if (soundll->in.buf)
 	o->free(o, soundll->in.buf);
     if (soundll->in.cnv.buf)
@@ -1168,6 +1185,20 @@ gensio_sound_ll_control(struct sound_ll *soundll, bool get, unsigned int option,
 	*datalen = gensio_pos_snprintf(data, *datalen, NULL, "sound");
 	return 0;
 
+    case GENSIO_CONTROL_LADDR:
+	if (!get)
+	    return GE_NOTSUP;
+	if (strcmp(data, "in") == 0) {
+	    *datalen = gensio_pos_snprintf(data, *datalen, NULL, "%s",
+					   soundll->in.cardname);
+	} else if (strcmp(data, "out") == 0) {
+	    *datalen = gensio_pos_snprintf(data, *datalen, NULL, "%s",
+					   soundll->out.cardname);
+	} else {
+	    return GE_NOTFOUND;
+	}
+	return 0;
+
     case GENSIO_CONTROL_IN_RATE:
 	if (!get)
 	    return GE_NOTSUP;
@@ -1359,7 +1390,8 @@ static struct sound_type *sound_types[] = {
 };
 
 static int
-setup_sound_info(struct gensio_os_funcs *o,
+setup_sound_info(struct gensio_pparm_info *p, const char *dir,
+		 struct gensio_os_funcs *o,
 		 struct sound_info *si, struct gensio_sound_info *io,
 		 bool isinput)
 {
@@ -1372,14 +1404,36 @@ setup_sound_info(struct gensio_os_funcs *o,
 		break;
 	}
     }
-    if (!sound_types[i])
+    if (!sound_types[i]) {
+	gensio_pparm_log(p, "%s: Unknown sound type: %s", dir, io->type);
 	return GE_INVAL;
+    }
 
     si->type = sound_types[i];
-    if (!io->devname || io->samplerate == 0 || io->chans == 0)
+    if (!io->devname) {
+	gensio_pparm_log(p, "%s: No device name", dir);
 	return GE_INVAL;
-    if (!io->format || io->bufsize == 0 || io->num_bufs == 0)
+    }
+    if (io->samplerate == 0) {
+	gensio_pparm_log(p, "%s: Sample rate is 0", dir);
 	return GE_INVAL;
+    }
+    if (io->chans == 0) {
+	gensio_pparm_log(p, "%s: Number of channnels is 0", dir);
+	return GE_INVAL;
+    }
+    if (!io->format) {
+	gensio_pparm_log(p, "%s: format is not set", dir);
+	return GE_INVAL;
+    }
+    if (io->bufsize == 0) {
+	gensio_pparm_log(p, "%s: Buffer size is 0", dir);
+	return GE_INVAL;
+    }
+    if (io->num_bufs == 0) {
+	gensio_pparm_log(p, "%s: Number of buffers is 0", dir);
+	return GE_INVAL;
+    }
 
     si->cnv.pfmt = GENSIO_SOUND_FMT_UNKNOWN;
     si->cnv.ufmt = GENSIO_SOUND_FMT_UNKNOWN;
@@ -1389,10 +1443,12 @@ setup_sound_info(struct gensio_os_funcs *o,
     si->samplerate = io->samplerate;
 
     err = setup_conv(io->format, io->pformat, si);
-    if (err)
+    if (err) {
+	gensio_pparm_log(p, "%s: Unknown format", dir);
 	return err;
+    }
 
-    err = si->type->setup(si, io);
+    err = si->type->setup(p, si, io);
     if (err)
 	return err;
 
@@ -1411,7 +1467,8 @@ setup_sound_info(struct gensio_os_funcs *o,
 }
 
 int
-gensio_sound_ll_alloc(struct gensio_os_funcs *o,
+gensio_sound_ll_alloc(struct gensio_pparm_info *p,
+		      struct gensio_os_funcs *o,
 		      struct gensio_sound_info *in,
 		      struct gensio_sound_info *out,
 		      struct gensio_ll **newll)
@@ -1424,8 +1481,10 @@ gensio_sound_ll_alloc(struct gensio_os_funcs *o,
     if (out && out->chans == 0)
 	out = NULL;
 
-    if (!in && !out)
+    if (!in && !out) {
+	gensio_pparm_slog(p, "Must set input or output channels");
 	return GE_INVAL;
+    }
 
     soundll = o->zalloc(o, sizeof(*soundll));
     if (!soundll)
@@ -1437,7 +1496,7 @@ gensio_sound_ll_alloc(struct gensio_os_funcs *o,
     if (in) {
 	soundll->in.is_input = true;
 	soundll->in.soundll = soundll;
-	err = setup_sound_info(o, &soundll->in, in, true);
+	err = setup_sound_info(p, "in", o, &soundll->in, in, true);
 	if (err)
 	    goto out_err;
     }
@@ -1445,7 +1504,7 @@ gensio_sound_ll_alloc(struct gensio_os_funcs *o,
     if (out) {
 	soundll->out.is_input = false;
 	soundll->out.soundll = soundll;
-	err = setup_sound_info(o, &soundll->out, out, false);
+	err = setup_sound_info(p, "out", o, &soundll->out, out, false);
 	if (err)
 	    goto out_err;
     }
