@@ -1386,6 +1386,7 @@ mux_new_channel(struct mux_data *muxdata, gensio_event cb, void *user_data,
 	goto out_free;
     gensio_set_is_packet(chan->io, true);
     gensio_set_is_reliable(chan->io, true);
+    gensio_set_is_mux(chan->io, true);
     if (gensio_is_authenticated(muxdata->child))
 	gensio_set_is_authenticated(chan->io, true);
     if (gensio_is_encrypted(muxdata->child))
@@ -1496,7 +1497,8 @@ muxc_alloc_channel_data(struct mux_data *muxdata,
 }
 
 static int
-gensio_mux_config(struct gensio_os_funcs *o,
+gensio_mux_config(struct gensio_pparm_info *p,
+		  struct gensio_os_funcs *o,
 		  const char * const args[],
 		  struct gensio_mux_config *data)
 {
@@ -1507,22 +1509,22 @@ gensio_mux_config(struct gensio_os_funcs *o,
     data->o = o;
 
     for (i = 0; args && args[i]; i++) {
-	if (gensio_check_keyds(args[i], "readbuf", &data->max_read_size) > 0)
+	if (gensio_pparm_ds(p, args[i], "readbuf", &data->max_read_size) > 0)
 	    continue;
-	if (gensio_check_keyds(args[i], "writebuf", &data->max_write_size) > 0)
+	if (gensio_pparm_ds(p, args[i], "writebuf", &data->max_write_size) > 0)
 	    continue;
-	if (gensio_check_keyboolv(args[i], "mode", "client", "server",
-				  &data->is_client) > 0)
+	if (gensio_pparm_boolv(p, args[i], "mode", "client", "server",
+			       &data->is_client) > 0)
 	    continue;
-	if (gensio_check_keyuint(args[i], "max_channels",
-				 &data->max_channels) > 0) {
+	if (gensio_pparm_uint(p, args[i], "max_channels",
+			      &data->max_channels) > 0) {
 	    if (data->max_channels > 65536 || data->max_channels < 1) {
 		rv = GE_INVAL;
 		goto out_err;
 	    }
 	    continue;
 	}
-	if (gensio_check_keyvalue(args[i], "service", &str) > 0) {
+	if (gensio_pparm_value(p, args[i], "service", &str) > 0) {
 	    data->service = gensio_strdup(o, str);
 	    if (!data->service)
 		goto out_err;
@@ -1586,6 +1588,8 @@ muxc_alloc_channel(struct mux_data *muxdata,
 {
     int err;
     struct gensio_mux_config data;
+    GENSIO_DECLARE_PPGENSIO(p, muxdata->o, ocdata->cb, "mux",
+			    ocdata->user_data);
 
     mux_lock(muxdata);
     if (muxdata->state != MUX_OPEN) {
@@ -1602,7 +1606,7 @@ muxc_alloc_channel(struct mux_data *muxdata,
     if (err)
 	return err;
 
-    err = gensio_mux_config(muxdata->o, ocdata->args, &data);
+    err = gensio_mux_config(&p, muxdata->o, ocdata->args, &data);
     if (err)
 	return err;
 
@@ -2792,6 +2796,7 @@ mux_gensio_alloc(struct gensio *child, const char *const args[],
     struct gensio_mux_config data;
     struct mux_data *muxdata;
     int ival;
+    GENSIO_DECLARE_PPGENSIO(p, o, cb, "mux", user_data);
 
     if (!gensio_is_reliable(child))
 	/* Cowardly refusing to run MUX over an unreliable connection. */
@@ -2811,7 +2816,7 @@ mux_gensio_alloc(struct gensio *child, const char *const args[],
     if (err)
 	return err;
 
-    err = gensio_mux_config(o, args, &data);
+    err = gensio_mux_config(&p, o, args, &data);
     if (err)
 	return err;
 
@@ -2823,6 +2828,7 @@ mux_gensio_alloc(struct gensio *child, const char *const args[],
     io = mux_chan0(muxdata)->io;
     gensio_set_is_packet(io, true);
     gensio_set_is_reliable(io, true);
+    gensio_set_is_mux(io, true);
     if (gensio_is_encrypted(child))
 	gensio_set_is_encrypted(io, true);
 
@@ -2839,7 +2845,8 @@ str_to_mux_gensio(const char *str, const char * const args[],
     int err;
     struct gensio *io2;
 
-    err = str_to_gensio(str, o, NULL, NULL, &io2);
+    /* cb is passed in for parmerr handling, it will be overriden later. */
+    err = str_to_gensio(str, o, cb, user_data, &io2);
     if (err)
 	return err;
 
@@ -2933,6 +2940,7 @@ mux_gensio_accepter_alloc(struct gensio_accepter *child,
 {
     struct muxna_data *nadata;
     int err, ival;
+    GENSIO_DECLARE_PPACCEPTER(p, o, cb, "mux", user_data);
 
     if (!gensio_acc_is_reliable(child))
 	/* Cowardly refusing to run MUX over an unreliable connection. */
@@ -2958,7 +2966,7 @@ mux_gensio_accepter_alloc(struct gensio_accepter *child,
 	o->free(o, nadata);
 	return err;
     }
-    err = gensio_mux_config(o, args, &nadata->data);
+    err = gensio_mux_config(&p, o, args, &nadata->data);
     if (err) {
 	o->free(o, nadata);
 	return err;
@@ -2973,6 +2981,7 @@ mux_gensio_accepter_alloc(struct gensio_accepter *child,
 	goto out_err;
     gensio_acc_set_is_packet(nadata->acc, true);
     gensio_acc_set_is_reliable(nadata->acc, true);
+    gensio_acc_set_is_mux(nadata->acc, true);
     *accepter = nadata->acc;
 
     return 0;
@@ -2992,7 +3001,8 @@ str_to_mux_gensio_accepter(const char *str, const char * const args[],
     int err;
     struct gensio_accepter *acc2 = NULL;
 
-    err = str_to_gensio_accepter(str, o, NULL, NULL, &acc2);
+    /* cb is passed in for parmerr handling, it will be overriden later. */
+    err = str_to_gensio_accepter(str, o, cb, user_data, &acc2);
     if (!err) {
 	err = mux_gensio_accepter_alloc(acc2, args, o, cb, user_data, acc);
 	if (err)
