@@ -116,6 +116,10 @@ gensio_unix_os_setupnewprog(void)
 	    return err;
 	}
 	free(groups);
+    } else {
+	err = setgroups(0, NULL);
+	if (err)
+	    return errno;
     }
 
     /* Sets the real, effective, and saved userid. */
@@ -1367,7 +1371,7 @@ get_logon_sid_from_token(HANDLE h, SID **logon_sid, bool *found)
     DWORD err;
     unsigned int i;
     TOKEN_GROUPS *grps = NULL;
-    SID *sid;
+    SID *sid = NULL;
 
     err = read_token_info(h, TokenGroups, (void **) &grps, NULL);
     if (err) {
@@ -2444,10 +2448,10 @@ set_flowcontrol(int fd, bool val)
     return ioctl(fd, TCXONC, val ? TCOOFF : TCOON);
 }
 
-static void
+static int
 do_break(int fd)
 {
-    ioctl(fd, TCSBRK, 0);
+    return ioctl(fd, TCSBRK, 0);
 }
 #else
 
@@ -2475,10 +2479,10 @@ set_flowcontrol(int fd, bool val)
     return tcflow(fd, val ? TCOOFF : TCOON);
 }
 
-static void
+static int
 do_break(int fd)
 {
-    tcsendbreak(fd, 0);
+    return tcsendbreak(fd, 0);
 }
 #endif
 
@@ -3007,10 +3011,13 @@ gensio_unix_termios_control(struct gensio_os_funcs *o, int op, bool get,
 	break;
 
     case GENSIO_IOD_CONTROL_SEND_BREAK:
-	if (get)
+	if (get) {
 	    *((int *) val) = 0;
-	else
-	    do_break(fd);
+	} else {
+	    rv = do_break(fd);
+	    if (rv)
+		rv = gensio_os_err_to_err(o, errno);
+	}
 	break;
 
     case GENSIO_IOD_CONTROL_DTR:
