@@ -43,8 +43,10 @@ struct stel_data {
     const struct gensio_telnet_filter_rops *rops;
     struct gensio_lock *lock;
 
-    bool allow_2217;
-    bool do_2217;
+    bool allow_rfc2217;
+    bool do_rfc2217;
+    bool allow_rfc1073;
+    bool do_rfc1073;
     bool cisco_baud;
     bool reported_modemstate;
     bool is_client;
@@ -129,7 +131,7 @@ stel_queue(struct stel_data *sdata, int option,
     struct stel_req *curr, *req;
     gensio_time timeout;
 
-    if (!sdata->do_2217)
+    if (!sdata->do_rfc2217)
 	return GE_NOTSUP;
 
     req = sdata->o->zalloc(sdata->o, sizeof(*req));
@@ -480,11 +482,11 @@ stelc_com_port_will_do(void *handler_data, unsigned char cmd)
 
     if (cmd == TN_DONT)
 	/* The remote end turned off RFC2217 handling. */
-	sdata->do_2217 = false;
+	sdata->do_rfc2217 = false;
     else
-	sdata->do_2217 = sdata->allow_2217;
+	sdata->do_rfc2217 = sdata->allow_rfc2217;
 
-    return sdata->do_2217;
+    return sdata->do_rfc2217;
 }
 
 static void
@@ -596,6 +598,31 @@ stelc_com_port_cmd(void *handler_data, const unsigned char *option,
     }
 }
 
+static int
+stelc_rfc1073_will_do(void *handler_data, unsigned char cmd)
+{
+    struct stel_data *sdata = handler_data;
+
+    if (cmd != TN_DO && cmd != TN_DONT)
+	/* We only handle these. */
+	return 0;
+
+    if (cmd == TN_DONT)
+	/* The remote end turned off RFC1073 handling. */
+	sdata->do_rfc1073 = false;
+    else
+	sdata->do_rfc1073 = sdata->allow_rfc1073;
+
+    return sdata->do_rfc1073;
+}
+
+static void
+stelc_rfc1073_cmd(void *handler_data, const unsigned char *option,
+		  unsigned int len)
+{
+    /* We don't get these. */
+}
+
 static void
 stelc_timeout(void *handler_data)
 {
@@ -673,6 +700,8 @@ struct gensio_telnet_filter_callbacks sergensio_telnet_filter_cbs = {
     .got_sync = stelc_got_sync,
     .com_port_will_do = stelc_com_port_will_do,
     .com_port_cmd = stelc_com_port_cmd,
+    .rfc1073_will_do = stelc_rfc1073_will_do,
+    .rfc1073_cmd = stelc_rfc1073_cmd,
     .timeout = stelc_timeout,
     .free = stel_free
 };
@@ -688,11 +717,11 @@ stels_cb_com_port_will_do(void *handler_data, unsigned char cmd)
     stel_lock(sdata);
     if (cmd == TN_WONT)
 	/* The remote end turned off RFC2217 handling. */
-	sdata->do_2217 = false;
+	sdata->do_rfc2217 = false;
     else
-	sdata->do_2217 = sdata->allow_2217;
+	sdata->do_rfc2217 = sdata->allow_rfc2217;
 
-    if (!sdata->reported_modemstate && sdata->do_2217) {
+    if (!sdata->reported_modemstate && sdata->do_rfc2217) {
 	struct gensio *io = sdata->io;
 
 	if (gensio_get_cb(io)) {
@@ -713,7 +742,7 @@ stels_cb_com_port_will_do(void *handler_data, unsigned char cmd)
     }
     stel_unlock(sdata);
 
-    return sdata->do_2217;
+    return sdata->do_rfc2217;
 }
 
 static void
@@ -849,6 +878,46 @@ stels_cb_com_port_cmd(void *handler_data, const unsigned char *option,
     }
 }
 
+static int
+stels_cb_rfc1073_will_do(void *handler_data, unsigned char cmd)
+{
+    struct stel_data *sdata = handler_data;
+
+    if (cmd != TN_WILL && cmd != TN_WONT)
+	/* We only handle these. */
+	return 0;
+
+    if (cmd == TN_WONT)
+	/* The remote end turned off RFC1073 handling. */
+	sdata->do_rfc1073 = false;
+    else
+	sdata->do_rfc1073 = sdata->allow_rfc1073;
+
+    return sdata->do_rfc1073;
+}
+
+static void
+stels_cb_rfc1073_cmd(void *handler_data, const unsigned char *option,
+		     unsigned int len)
+{
+    struct stel_data *sdata = handler_data;
+    struct gensio *io = sdata->io;
+    char buf[30];
+    gensiods buflen;
+    unsigned int width, height;
+
+    if (len < 5)
+	return;
+    option++; /* Skip the option number. */
+    width = (option[0] << 8) | option[1];
+    height = (option[2] << 8) | option[3];
+    buflen = snprintf(buf, sizeof(buf), "%u:%u", height, width);
+    if (buflen >= sizeof(buf))
+	buflen = sizeof(buf) - 1;
+    gensio_cb(io, GENSIO_EVENT_WIN_SIZE, 0,
+	      (unsigned char *) buf, &buflen, NULL);
+}
+
 static void
 stels_got_cmd(void *handler_data, unsigned char cmd)
 {
@@ -874,7 +943,7 @@ stels_timeout(void *handler_data)
     struct stel_data *sdata = handler_data;
 
     stel_lock(sdata);
-    if (!sdata->reported_modemstate && sdata->do_2217) {
+    if (!sdata->reported_modemstate && sdata->do_rfc2217) {
 	struct gensio *io = sdata->io;
 	int val = 255;
 	gensiods vlen = sizeof(val);
@@ -899,17 +968,21 @@ struct gensio_telnet_filter_callbacks sergensio_telnet_server_filter_cbs = {
     .got_cmd = stels_got_cmd,
     .com_port_will_do = stels_cb_com_port_will_do,
     .com_port_cmd = stels_cb_com_port_cmd,
+    .rfc1073_will_do = stels_cb_rfc1073_will_do,
+    .rfc1073_cmd = stels_cb_rfc1073_cmd,
     .timeout = stels_timeout,
     .free = stel_free
 };
 
 static int
-stel_setup(const char * const args[], bool default_is_client,
+stel_setup(struct gensio_pparm_info *p,
+	   const char * const args[], bool default_is_client,
 	   struct gensio_os_funcs *o, struct stel_data **rsdata)
 {
     struct stel_data *sdata;
     unsigned int i;
-    bool allow_2217 = false;
+    bool allow_rfc2217 = false;
+    bool allow_rfc1073 = false;
     bool is_client = default_is_client;
     int err;
     int rv, ival;
@@ -918,30 +991,41 @@ stel_setup(const char * const args[], bool default_is_client,
 			    GENSIO_DEFAULT_BOOL, NULL, &ival);
     if (rv)
 	return rv;
-    allow_2217 = ival;
+    allow_rfc2217 = ival;
+
+    rv = gensio_get_default(o, "telnet", "winsize", false,
+			    GENSIO_DEFAULT_BOOL, NULL, &ival);
+    if (rv)
+	return rv;
+    allow_rfc1073 = ival;
 
     for (i = 0; args && args[i]; i++) {
-	if (gensio_check_keybool(args[i], "rfc2217", &allow_2217) > 0)
+	if (gensio_pparm_bool(p, args[i], "rfc2217", &allow_rfc2217) > 0)
 	    continue;
-	if (gensio_check_keyboolv(args[i], "mode", "client", "server",
-				  &is_client) > 0)
+	if (gensio_pparm_bool(p, args[i], "winsize", &allow_rfc1073) > 0)
+	    continue;
+	if (gensio_pparm_boolv(p, args[i], "mode", "client", "server",
+			       &is_client) > 0)
 	    continue;
 	/* Ignore everything else, the filter will handle it. */
     }
+    if (p->err)
+	return GE_INVAL;
 
     sdata = o->zalloc(o, sizeof(*sdata));
     if (!sdata)
 	return GE_NOMEM;
 
     sdata->o = o;
-    sdata->allow_2217 = allow_2217;
+    sdata->allow_rfc2217 = allow_rfc2217;
+    sdata->allow_rfc1073 = allow_rfc1073;
     sdata->is_client = is_client;
 
     sdata->lock = o->alloc_lock(o);
     if (!sdata->lock)
 	goto out_nomem;
 
-    err = gensio_telnet_filter_alloc(o, args, true,
+    err = gensio_telnet_filter_alloc(p, o, args, true,
 				     (is_client ?
 				      &sergensio_telnet_filter_cbs :
 				      &sergensio_telnet_server_filter_cbs),
@@ -977,8 +1061,9 @@ telnet_gensio_alloc(struct gensio *child, const char * const args[],
     struct gensio_ll *ll = NULL;
     struct gensio *io = NULL;
     int err;
+    GENSIO_DECLARE_PPGENSIO(p, o, cb, "telnet", user_data);
 
-    err = stel_setup(args, true, o, &sdata);
+    err = stel_setup(&p, args, true, o, &sdata);
     if (err)
 	return err;
 
@@ -994,7 +1079,7 @@ telnet_gensio_alloc(struct gensio *child, const char * const args[],
 
     sdata->io = io;
 
-    if (sdata->allow_2217) {
+    if (sdata->allow_rfc2217) {
 	err = sergensio_addclass(o, io, sergensio_stel_func, sdata,
 				 &sdata->sio);
 	if (err)
@@ -1032,7 +1117,8 @@ str_to_telnet_gensio(const char *str, const char * const args[],
     int err;
     struct gensio *io2;
 
-    err = str_to_gensio(str, o, NULL, NULL, &io2);
+    /* cb is passed in for parmerr handling, it will be overriden later. */
+    err = str_to_gensio(str, o, cb, user_data, &io2);
     if (err)
 	return err;
 
@@ -1050,8 +1136,11 @@ struct stela_data {
     gensiods max_write_size;
 
     struct gensio_os_funcs *o;
+    gensio_accepter_event cb;
+    void *user_data;
 
-    bool allow_2217;
+    bool allow_rfc2217;
+    bool allow_rfc1073;
     bool is_client;
 };
 
@@ -1071,30 +1160,38 @@ stela_alloc_gensio(void *acc_data, const char * const *iargs,
 {
     struct stela_data *stela = acc_data;
     struct gensio_os_funcs *o = stela->o;
-    const char *args[5] = {NULL, NULL, NULL, NULL, NULL};
+    const char *args[6] = { NULL, NULL, NULL, NULL, NULL, NULL };
     char buf1[50], buf2[50];
     unsigned int i;
-    bool allow_2217 = stela->allow_2217;
+    bool allow_rfc2217 = stela->allow_rfc2217;
+    bool allow_rfc1073 = stela->allow_rfc1073;
     gensiods max_write_size = stela->max_write_size;
     gensiods max_read_size = stela->max_read_size;
     bool is_client = stela->is_client;
+    GENSIO_DECLARE_PPACCEPTER(p, stela->o, stela->cb, "telnet",
+			      stela->user_data);
 
     for (i = 0; iargs && iargs[i]; i++) {
-	if (gensio_check_keybool(iargs[i], "rfc2217", &allow_2217) > 0)
+	if (gensio_pparm_bool(&p, iargs[i], "rfc2217", &allow_rfc2217) > 0)
 	    continue;
-	if (gensio_check_keyds(iargs[i], "writebuf", &max_write_size) > 0)
+	if (gensio_pparm_bool(&p, iargs[i], "winsize", &allow_rfc1073) > 0)
 	    continue;
-	if (gensio_check_keyds(iargs[i], "readbuf", &max_read_size) > 0)
+	if (gensio_pparm_ds(&p, iargs[i], "writebuf", &max_write_size) > 0)
 	    continue;
-	if (gensio_check_keyboolv(iargs[i], "mode", "client", "server",
-				  &is_client) > 0)
+	if (gensio_pparm_ds(&p, iargs[i], "readbuf", &max_read_size) > 0)
 	    continue;
+	if (gensio_pparm_boolv(&p, iargs[i], "mode", "client", "server",
+			       &is_client) > 0)
+	    continue;
+	gensio_pparm_unknown_parm(&p, iargs[i]);
 	return GE_INVAL;
     }
 
     i = 0;
-    if (allow_2217)
+    if (allow_rfc2217)
 	args[i++] = "rfc2217=true";
+    if (allow_rfc1073)
+	args[i++] = "winsize=true";
     if (max_read_size != GENSIO_DEFAULT_BUF_SIZE) {
 	snprintf(buf1, sizeof(buf1), "readbuf=%lu",
 		 (unsigned long) max_read_size);
@@ -1119,18 +1216,21 @@ stela_new_child(void *acc_data, void **finish_data,
     struct gensio_os_funcs *o = stela->o;
     struct stel_data *sdata;
     int err;
-    char arg1[25], arg2[25], arg3[25], arg4[25];
-    const char *args[5] = { arg1, arg2, arg3, arg4, NULL };
+    char arg1[25], arg2[25], arg3[25], arg4[25], arg5[25];
+    const char *args[6] = { arg1, arg2, arg3, arg4, arg5, NULL };
+    GENSIO_DECLARE_PPACCEPTER(p, stela->o, stela->cb, "telnet",
+			      stela->user_data);
 
-    snprintf(arg1, sizeof(arg1), "rfc2217=%d", stela->allow_2217);
-    snprintf(arg2, sizeof(arg2), "writebuf=%lu",
+    snprintf(arg1, sizeof(arg1), "rfc2217=%d", stela->allow_rfc2217);
+    snprintf(arg2, sizeof(arg2), "winsize=%d", stela->allow_rfc1073);
+    snprintf(arg3, sizeof(arg3), "writebuf=%lu",
 	     (unsigned long) stela->max_write_size);
-    snprintf(arg3, sizeof(arg3), "readbuf=%lu",
+    snprintf(arg4, sizeof(arg4), "readbuf=%lu",
              (unsigned long) stela->max_read_size);
-    snprintf(arg4, sizeof(arg4), "mode=%s",
+    snprintf(arg5, sizeof(arg5), "mode=%s",
 	     stela->is_client ? "client" : "server");
 
-    err = stel_setup(args, false, o, &sdata);
+    err = stel_setup(&p, args, false, o, &sdata);
     if (err)
 	return err;
 
@@ -1149,7 +1249,7 @@ stela_finish_parent(void *acc_data, void *finish_data, struct gensio *io,
 
     sdata->io = io;
 
-    if (sdata->allow_2217) {
+    if (sdata->allow_rfc2217) {
 	err = sergensio_addclass(sdata->o, io, sergensio_stel_func, sdata,
 				 &sdata->sio);
 	if (err)
@@ -1204,27 +1304,38 @@ telnet_gensio_accepter_alloc(struct gensio_accepter *child,
     unsigned int i;
     gensiods max_read_size = GENSIO_DEFAULT_BUF_SIZE;
     gensiods max_write_size = GENSIO_DEFAULT_BUF_SIZE;
-    bool allow_2217 = false;
+    bool allow_rfc2217 = false;
+    bool allow_rfc1073 = false;
     bool is_client = false;
     struct gensio_accepter *accepter = NULL;
     int rv, ival;
+    GENSIO_DECLARE_PPACCEPTER(p, o, cb, "telnet", user_data);
 
     rv = gensio_get_default(o, "telnet", "rfc2217", false,
 			    GENSIO_DEFAULT_BOOL, NULL, &ival);
     if (rv)
 	return rv;
-    allow_2217 = ival;
+    allow_rfc2217 = ival;
+
+    rv = gensio_get_default(o, "telnet", "winsize", false,
+			    GENSIO_DEFAULT_BOOL, NULL, &ival);
+    if (rv)
+	return rv;
+    allow_rfc1073 = ival;
 
     for (i = 0; args && args[i]; i++) {
-	if (gensio_check_keybool(args[i], "rfc2217", &allow_2217) > 0)
+	if (gensio_pparm_bool(&p, args[i], "rfc2217", &allow_rfc2217) > 0)
 	    continue;
-	if (gensio_check_keyds(args[i], "writebuf", &max_write_size) > 0)
+	if (gensio_pparm_bool(&p, args[i], "winsize", &allow_rfc1073) > 0)
 	    continue;
-	if (gensio_check_keyds(args[i], "readbuf", &max_read_size) > 0)
+	if (gensio_pparm_ds(&p, args[i], "writebuf", &max_write_size) > 0)
 	    continue;
-	if (gensio_check_keyboolv(args[i], "mode", "client", "server",
-				  &is_client) > 0)
+	if (gensio_pparm_ds(&p, args[i], "readbuf", &max_read_size) > 0)
 	    continue;
+	if (gensio_pparm_boolv(&p, args[i], "mode", "client", "server",
+			       &is_client) > 0)
+	    continue;
+	gensio_pparm_unknown_parm(&p, args[i]);
 	return GE_INVAL;
     }
 
@@ -1233,9 +1344,12 @@ telnet_gensio_accepter_alloc(struct gensio_accepter *child,
 	return GE_NOMEM;
 
     stela->o = o;
+    stela->cb = cb;
+    stela->user_data = user_data;
     stela->max_write_size = max_write_size;
     stela->max_read_size = max_read_size;
-    stela->allow_2217 = allow_2217;
+    stela->allow_rfc2217 = allow_rfc2217;
+    stela->allow_rfc1073 = allow_rfc1073;
     stela->is_client = is_client;
 
     err = gensio_gensio_accepter_alloc(child, o, "telnet",
@@ -1245,7 +1359,7 @@ telnet_gensio_accepter_alloc(struct gensio_accepter *child,
     if (err)
 	goto out_err;
 
-    if (allow_2217) {
+    if (allow_rfc2217) {
 	err = sergensio_acc_addclass(o, accepter, sergensio_stela_func, stela,
 				     &stela->sacc);
 	if (err)
@@ -1275,7 +1389,8 @@ str_to_telnet_gensio_accepter(const char *str, const char * const args[],
     int err;
     struct gensio_accepter *acc2 = NULL;
 
-    err = str_to_gensio_accepter(str, o, NULL, NULL, &acc2);
+    /* cb is passed in for parmerr handling, it will be overriden later. */
+    err = str_to_gensio_accepter(str, o, cb, user_data, &acc2);
     if (!err) {
 	err = telnet_gensio_accepter_alloc(acc2, args, o, cb, user_data, acc);
 	if (err)
