@@ -194,6 +194,16 @@ class EventHandler:
         """
         return ""
 
+    def win_size(self, io, height, width):
+        """Called to report a window size from the remote end of the
+        connection.
+
+        height -- The number of rows being reported
+        width -- The number of columns being reported
+
+        """
+        return
+
     def signature(self, sio):
         """Called from the telnet server to report that the client has
         requested the RFC2217 signature.  Call sg_signature(signature,
@@ -364,6 +374,13 @@ class EventHandler:
             SERGENSIO_LINESTATE_TIMEOUT_ERR.
         """
 
+    def parmlog(self, log):
+        """Called during parsing of gensio strings, this reports
+        more detailed errors about what went wrong in parsing.
+
+        log -- The detailed log of the parsing error.
+        """
+
 class OpenDone:
 
         """A template for a class handling the finish of an open."""
@@ -930,6 +947,13 @@ class AccEventHandler:
         """
         return "password"
 
+    def parmlog(self, log):
+        """Called during parsing of gensio strings, this reports
+        more detailed errors about what went wrong in parsing.
+
+        log -- The detailed log of the parsing error.
+        """
+
 class ShutdownDone:
     """A class template for receiving shutdown done reports."""
 
