@@ -9,9 +9,9 @@ import sys
 from utils import *
 import gensio
 import ipmisimdaemon
-import termios
-from termioschk import *
 try:
+    import termios
+    from termioschk import *
     from serialsim import *
 except:
     sys.exit(77)
@@ -45,8 +45,7 @@ def check_baud_set(speed, bspeed):
     c = compare_termios(t, io1_r_termios);
     if c != -1:
         raise Exception("Termios %d failure on item %d" % (speed, c))
-    utils.io_close(io1)
-    utils.io_close(io2)
+    utils.io_close((io1, io2))
     isim.terminate()
 
 check_pipe_dev()
@@ -91,8 +90,8 @@ if (io1.handler.wait_timeout(1000) == 0):
 
 print("Testing flush")
 # Flush is hard to test, just make sure it doesn't crash.
-sio2.sg_flush(gensio.SERGIO_FLUSH_RCV_BUFFER)
-sio2.sg_flush(gensio.SERGIO_FLUSH_XMIT_BUFFER)
+sio2.sg_flush(gensio.SERGENSIO_FLUSH_RCV_BUFFER)
+sio2.sg_flush(gensio.SERGENSIO_FLUSH_XMIT_BUFFER)
 
 print("Testing CTS")
 io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_CTS_CHANGED |
@@ -127,8 +126,7 @@ if (io1.handler.wait_timeout(3000) == 0):
 
 # No easy way to test ring.
 
-io_close(io1)
-io_close(io2)
+io_close((io1, io2))
 isim.terminate()
 
 print("Testing deassertion of CTS, DCD, and DSR at start");
@@ -198,8 +196,7 @@ sio1.sg_modemstate(gensio.SERGENSIO_MODEMSTATE_CTS |
 if (io1.handler.wait_timeout(3000) == 0):
     raise Exception("Timed out waiting for DCD/DSR on")
 
-io_close(io1)
-io_close(io2)
+io_close((io1, io2))
 del io1
 del io2
 del sio1
