@@ -80,6 +80,9 @@ class HandleData:
         self.compared_oob = 0
         self.to_compare_oob = None
         self.to_waitfor = None
+        self.expecting_winsize = False
+        self.expected_winsize_height = -1
+        self.expected_winsize_width = -1
         self.expecting_modemstate = False
         self.expecting_linestate = False
         self.expecting_remclose = expect_remclose
@@ -359,6 +362,28 @@ class HandleData:
         self.expected_modemstate = modemstate
         return
 
+    def win_size(self, io, height, width):
+        if (not self.expecting_winsize):
+            if (debug or self.debug):
+                print("Got unexpected window size for %s: %d %d" %
+                      (self.name, height, width))
+            return
+        if (height != self.expected_winsize_height or
+            width != self.expected_winsize_width):
+            raise HandlerException(
+                "%s: Expecting window size %d:%d, got %d:%d" %
+                (self.name, self.expected_winsize_height,
+                 self.expected_winsize_width, height, width))
+        self.expecting_winsize = False
+        self.waiter.wake()
+        return
+
+    def set_expected_win_size(self, height, width):
+        self.expecting_winsize = True
+        self.expected_winsize_height = height
+        self.expected_winsize_width = width
+        return
+
     def linestate(self, io, linestate):
         if (not self.expecting_linestate):
             if (debug or self.debug):
@@ -741,7 +766,7 @@ class TestAccept:
                  expected_raddr = None, expected_acc_laddr = None,
                  chunksize = 10240, get_port = True, except_on_log = False,
                  is_sergensio = False, enable_oob = False, timeout = 0,
-                 close_timeout = 1000):
+                 close_timeout = 1000, enable_read_io1 = False):
         self.o = o
         self.io1 = None
         self.io2 = None
@@ -805,6 +830,10 @@ class TestAccept:
                 del io1.handler.io
                 del io1.handler
                 raise
+            if enable_read_io1:
+                # Some gensios, like telnet, need to have read enabled
+                # on the client for the accept to complete.
+                io1.read_cb_enable(True);
             if expected_raddr:
                 check_raddr(io1, self.name, expected_raddr)
             if (io1_dummy_write):
@@ -816,6 +845,8 @@ class TestAccept:
             if (self.wait_timeout(1000) == 0):
                 raise Exception(("%s: %s: " % ("test_accept", self.name)) +
                                 ("Timed out waiting for initial connection"))
+            if enable_read_io1:
+                io1.read_cb_enable(False);
             if (io1_dummy_write):
                 self.io2.handler.set_compare(io1_dummy_write)
                 if (self.io2.handler.wait_timeout(1000) == 0):
